@@ -496,7 +496,13 @@ namespace Tomboy
 
 			Logger.Log ("Saving '{0}'...", data.Data.Title);
 
-			NoteArchiver.Write (filepath, data.GetDataSynchronized ());
+			try {
+				NoteArchiver.Write (filepath, data.GetDataSynchronized ());
+			} catch (Exception e) {
+				// Probably IOException or UnauthorizedAccessException?
+				Logger.Error ("Exception while saving note: " + e.ToString ());
+				NoteUtils.ShowIOErrorDialog (window);
+			}
 
 			if (Saved != null)
 				Saved (this);
@@ -778,30 +784,12 @@ namespace Tomboy
 		/// </summary>
 		public string GetCompleteNoteXml ()
 		{
-			if (!File.Exists (filepath))
-				return null;
-
-			// Make sure file contents are up to date
-			save_needed = true; // HACK: Catches newly created notes
-			Save ();
-
-			StreamReader reader = null;
-			try {
-				reader = new StreamReader (filepath);
-				return reader.ReadToEnd ();
-			} catch (Exception e) {
-				Logger.Error ("Error received while attempting to read " +
-				              filepath + ": " + e.Message);
-				return null;
-			} finally {
-				if (reader != null)
-					reader.Close ();
-			}
+			return NoteArchiver.WriteString (data.GetDataSynchronized ());
 		}
 
 		// Reload note data from a complete note XML string
 		// Should referesh note window, too
-		public void LoadForeignNoteXml (string foreignNoteXml)
+		public void LoadForeignNoteXml (string foreignNoteXml, ChangeType changeType)
 		{
 			if (foreignNoteXml == null)
 				throw new ArgumentNullException ("foreignNoteXml");
@@ -864,7 +852,8 @@ namespace Tomboy
 
 			xml.Close ();
 
-			// TODO: Any reason to queue a save here?  Maybe not for sync but for others?
+			// Allow method caller to specify ChangeType (mostly needed by sync)
+			QueueSave (changeType);
 		}
 
 		// TODO: CODE DUPLICATION SUCKS
@@ -1237,6 +1226,16 @@ namespace Tomboy
 			return note;
 		}
 
+		public static string WriteString(NoteData note)
+		{
+			StringWriter str = new StringWriter ();
+			XmlTextWriter xml = new XmlTextWriter (str);
+			Instance.Write (xml, note);
+			xml.Close ();
+			str.Flush();
+			return str.ToString ();
+		}
+
 		public static void Write (string write_file, NoteData note)
 		{
 			Instance.WriteFile (write_file, note);
@@ -1465,5 +1464,23 @@ namespace Tomboy
 
 			dialog.Destroy();
 		}
+		
+		public static void ShowIOErrorDialog (Gtk.Window parent)
+		{
+			HIGMessageDialog dialog =
+				new HIGMessageDialog (
+				                      parent,
+				                      Gtk.DialogFlags.DestroyWithParent,
+				                      Gtk.MessageType.Error,
+				                      Gtk.ButtonsType.Ok,
+				                      Catalog.GetString ("Error saving note data."),
+				                      Catalog.GetString ("An error occurred while saving your notes. " +
+				                                         "Please check that you have sufficient disk " +
+				                                         "space, and that you have appropriate rights " +
+				                                         "on ~/.tomboy. Error details can be found in " +
+				                                         "~/.tomboy.log."));
+			dialog.Run ();
+			dialog.Destroy ();
+		}
 	}
 }
